//#########################################################
//#
//# Titre : 	Utilitaires Liste Chainee et CHAT LINUX Automne 13
//#			SIF-1015 - Systeme d'exploitation
//#			Universite du Quebec a Trois-Rivieres
//#
//# Auteur : 	Francois Meunier
//#	Date :	Septembre 2013
//#
//# Langage : 	ANSI C on LINUX 
//#
//#######################################

#include "gestionListeChaineeCHAT.h"

//Pointeur de tete de liste
extern struct noeud* head;
//Pointeur de queue de liste pour ajout rapide
extern struct noeud* queue;


//#######################################
//#
//# Recherche le PREDECESSEUR d'un item dans la liste chainee
//#
//# RETOUR: Le pointeur vers le predecesseur est retourne
//# 		
//#			
//# 		Retourne NULL dans le cas ou l'item est introuvable
//#
struct noeud * findPrevCHAT(const char* ptrNick, const char* ptrGroupe){

	//La liste est vide 
	if ((head==NULL)&&(queue==NULL)) return NULL;

	//Pointeur de navigation
	struct noeud * ptr = head;

	//Tant qu'un item suivant existe
	while (ptr->suivant!=NULL){

		//Est-ce le predecesseur de l'item recherche?
		if((strcmp(ptr->suivant->membre.ptrNick,ptrNick) == 0) && (strcmp(ptr->suivant->membre.ptrGroupe,ptrGroupe) == 0) ){
		
			//On retourne un pointeur sur l'item precedent
			return ptr;
		}

		//Deplacement du pointeur de navigation
		ptr=ptr->suivant;
		}

	//On retourne un pointeur NULL
	return NULL;
	}

//#######################################
//#
//# Ajoute un item dans la liste chainee
//#
void addItemCHAT(const char* ptrNick, const char* ptrGroupe){

	//Creation de l'enregistrement en memoire
	struct noeud* ni = (struct noeud*)malloc(sizeof(struct noeud));


	//Affectation des valeurs des champs
	strcpy(ni->membre.ptrNick, ptrNick);
	strcpy(ni->membre.ptrGroupe, ptrGroupe);


	if(head == NULL) // ajout au debut de la liste vide
	{
		//  premier noeud 
		ni->suivant= NULL;
		queue = head = ni;

	}
	else  // ajout a la fin de la liste
	{
		struct noeud* tptr = queue;
		ni->suivant= NULL;
		queue = ni;
		tptr->suivant = ni;
	}
	
}

//#######################################
//#
//# Modifie un item de la liste chainee
//#
void modifyItemCHAT(const int noNickGroupe, const char* ptrNick, const char* ptrGroupe){

	int noentree=1;
	//Verification sommaire (noNickGroupe>0 et liste non vide)

	if ((noNickGroupe<1)||((head==NULL)&&(queue==NULL)))
		return;


	//Recherche de l'element a� modifier
	

	struct noeud * ptr = head;			//premier element


	while (ptr!=NULL){

		//Element a modifier
		if (noentree==noNickGroupe){
			//Affectation des valeurs des champs
			strcpy(ptr->membre.ptrNick, ptrNick);
			strcpy(ptr->membre.ptrGroupe, ptrGroupe);
			//printf("%d: %s \t\t  %s\n",noentree,ptr->membre.ptrNick,ptr->membre.ptrGroupe);
			return;
			}
		
		else{
			ptr = ptr->suivant;
			noentree++;
		}

	}

}


//#######################################
//#
//# Retire un item de la liste chainee
//#
void removeItemCHAT(const char* ptrNick, const char* ptrGroupe){

	struct noeud * ptr;
	struct noeud * optr;

	//Verification sommaire liste non vide
	
	if ((head==NULL)&&(queue==NULL)) 
		return;

	
	​if((strcmp(head->membre.ptrNick,ptrNick) == 0) && (strcmp(head->membre.ptrGroupe,ptrGroupe) == 0) )
    {
        ptr = head; // suppression du premier element de la liste

        if(head == ptr) // suppression de l'element de tete
        {
            if(head==queue) // un seul element dans la liste
            {
                free(ptr);
                queue = head = NULL;
                return;
            }
            head = ptr->suivant;
            //printf("tete\n");
            free(ptr);
        }
    }
    else
    {
        ptr = findPrevCHAT(ptrNick,ptrGroupe);

        //Item  trouve
        if (ptr!=NULL)
        {
            if (queue==ptr->suivant) // suppression de l'element de queue
            {
                queue=ptr;
                free(ptr->suivant);
                ptr->suivant=NULL;
                //printf("queue\n");
                return;
            }
            else // suppression d'un element dans la liste
            {
                optr = ptr->suivant;
                ptr->suivant = ptr->suivant->suivant;
                //printf("autre\n");
                free(optr);
            }
        }
    }​
	
	
}


//#######################################
//#
//# Affiche les items dont le numero sequentiel est compris dans un intervalle
//#
void listItemsCHAT(const int start, const int end){

	int noentree=1;

	//Affichage des entetes de colonnes
	printf("\n======================================================\n");
	printf("NICK		GROUPE                                          \n");
	printf("======================================================\n");

	struct noeud * ptr = head;			//premier element


	while (ptr!=NULL){

		//L'item a un numero sequentiel dans l'interval defini
		if ((noentree>=start)&&(noentree<=end)){
			printf("%d: %s \t\t  %s\n",noentree,
				ptr->membre.ptrNick,
				ptr->membre.ptrGroupe);
			}
		if (noentree>end){
			//L'ensemble des items potentiels sont maintenant passes
			//Deplacement immediatement a� la FIN de la liste
			//Notez que le pointeur optr est toujours valide
			ptr=NULL;
			}
		else{
			ptr = ptr->suivant;
			noentree++;
		}

	}

	//Affichage des pieds de colonnes
	printf("======================================================\n\n");
}

//#######################################
//#
//# Transmission d'un message aux membres d'un groupe 
//#
void transTextCHAT(const char* ptrNick, const char* ptrGroupe, const char* ptrTexte){


	struct noeud * ptr = head;			//premier element


	printf("%s \t %s ",ptrNick, ptrGroupe);
	printf("TEXTE ENVOYE: %s\n",ptrTexte);

	while (ptr!=NULL){
		
		if((strcmp(ptr->membre.ptrNick,ptrNick) != 0) && (strcmp(ptr->membre.ptrGroupe,ptrGroupe) == 0) )
		{						
				printf("%s \t %s ",ptr->membre.ptrNick, ptr->membre.ptrGroupe);
				printf("TEXTE RECU: %s\n",ptrTexte);
		}
		ptr = ptr->suivant;

	}

}